@@ -36,6 +36,7 @@
 #include "ngtcp2/ngtcp2_crypto_wolfssl.h"
 
 #include "util.h"
+#include "shared.h"
 #include "debug.h"
 
 using namespace std::literals;
@@ -94,11 +95,12 @@ uint64_t LinkConfig::compute_expected_goodput(Timestamp::duration rtt) const {
     return g;
   }
 
+  constexpr double margin = 0.95;
+
   return std::min(g,
-                  static_cast<uint64_t>(
-                    static_cast<double>(MAX_UDP_PAYLOAD_SIZE * NGTCP2_SECONDS /
-                                        static_cast<uint64_t>(rtt.count())) /
-                    sqrt(loss) * 8));
+                  static_cast<uint64_t>(MAX_UDP_PAYLOAD_SIZE * NGTCP2_SECONDS /
+                                        static_cast<double>(rtt.count()) /
+                                        sqrt(loss) * 8 * margin));
 }
 
 namespace {
@@ -181,8 +183,7 @@ ngtcp2_transport_params default_server_transport_params() {
   return params;
 }
 
-std::tuple<ngtcp2_sockaddr_union, ngtcp2_socklen> getaddrinfo(const char *host,
-                                                              const char *svc) {
+Sockaddr getaddrinfo(const char *host, const char *svc) {
   auto hints = addrinfo{
     .ai_flags = AI_NUMERICHOST | AI_NUMERICSERV,
     .ai_family = AF_UNSPEC,
@@ -193,32 +194,29 @@ std::tuple<ngtcp2_sockaddr_union, ngtcp2_socklen> getaddrinfo(const char *host,
   (void)rv;
   assert(0 == rv);
 
-  ngtcp2_sockaddr_union un;
-
-  memcpy(&un, rp->ai_addr, rp->ai_addrlen);
-
-  auto addrlen = rp->ai_addrlen;
+  Sockaddr skaddr;
+  sockaddr_set(skaddr, rp->ai_addr);
 
   freeaddrinfo(rp);
 
-  return {un, addrlen};
+  return skaddr;
 }
 
 ngtcp2_addr default_client_addr() {
-  static auto [un, addrlen] = getaddrinfo("10.0.1.1", "12345");
+  static auto skaddr = getaddrinfo("10.0.1.1", "12345");
 
   return ngtcp2_addr{
-    .addr = &un.sa,
-    .addrlen = addrlen,
+    .addr = as_sockaddr(skaddr),
+    .addrlen = sockaddr_size(skaddr),
   };
 }
 
 ngtcp2_addr default_server_addr() {
-  static auto [un, addrlen] = getaddrinfo("10.0.2.1", "443");
+  static auto skaddr = getaddrinfo("10.0.2.1", "443");
 
   return ngtcp2_addr{
-    .addr = &un.sa,
-    .addrlen = addrlen,
+    .addr = as_sockaddr(skaddr),
+    .addrlen = sockaddr_size(skaddr),
   };
 }
 
@@ -512,35 +510,18 @@ int Endpoint::on_timeout(const Context &ctx) {
 }
 
 NetworkPath to_network_path(const ngtcp2_path *path) {
-  NetworkPath res{
-    .local =
-      {
-        .len = path->local.addrlen,
-      },
-    .remote =
-      {
-        .len = path->remote.addrlen,
-      },
-  };
+  NetworkPath res;
 
-  memcpy(&res.local.su, path->local.addr, res.local.len);
-  memcpy(&res.remote.su, path->remote.addr, res.remote.len);
+  res.local.set(path->local.addr);
+  res.remote.set(path->remote.addr);
 
   return res;
 }
 
 ngtcp2_path to_ngtcp2_path(const NetworkPath &path) {
   return {
-    .local =
-      {
-        .addr = const_cast<ngtcp2_sockaddr *>(&path.local.su.sa),
-        .addrlen = path.local.len,
-      },
-    .remote =
-      {
-        .addr = const_cast<ngtcp2_sockaddr *>(&path.remote.su.sa),
-        .addrlen = path.remote.len,
-      },
+    .local = as_ngtcp2_addr(path.local),
+    .remote = as_ngtcp2_addr(path.remote),
   };
 }
 
