@@ -37,7 +37,7 @@ using namespace ngtcp2;
 
 class TLSServerContext {
 public:
-  TLSServerContext();
+  TLSServerContext() = default;
   ~TLSServerContext();
 
   int init(const char *private_key_file, const char *cert_file,
@@ -48,7 +48,7 @@ public:
   void enable_keylog();
 
 private:
-  SSL_CTX *ssl_ctx_;
+  SSL_CTX *ssl_ctx_{};
 };
 
 #endif // !defined(TLS_SERVER_CONTEXT_BORINGSSL_H)
