@@ -40,6 +40,7 @@
 
 #include <cassert>
 #include <cstring>
+#include <cstdlib>
 #include <chrono>
 #include <array>
 #include <iostream>
@@ -351,15 +352,8 @@ std::string straddr(const sockaddr *sa, socklen_t salen) {
   return res;
 }
 
-uint16_t port(const sockaddr_union *su) {
-  switch (su->sa.sa_family) {
-  case AF_INET:
-    return ntohs(su->in.sin_port);
-  case AF_INET6:
-    return ntohs(su->in6.sin6_port);
-  default:
-    return 0;
-  }
+std::string straddr(const Address &addr) {
+  return straddr(addr.as_sockaddr(), addr.size());
 }
 
 bool prohibited_port(uint16_t port) {
@@ -867,6 +861,18 @@ std::span<uint64_t, 2> generate_siphash_key() {
   return key;
 }
 
+std::string realpath(const char *path) {
+  auto cpath = ::realpath(path, nullptr);
+  if (!cpath) {
+    assert(0);
+    abort();
+  }
+
+  auto cpath_d = defer([cpath] { free(cpath); });
+
+  return cpath;
+}
+
 } // namespace util
 
 std::ostream &operator<<(std::ostream &os, const ngtcp2_cid &cid) {
