@@ -112,19 +112,6 @@ typedef enum {
    packet as much as possible if the packet is not empty. */
 #define NGTCP2_WRITE_PKT_FLAG_PADDING_IF_NOT_EMPTY 0x08u
 
-/*
- * ngtcp2_max_frame is defined so that it covers the largest ACK
- * frame.
- */
-typedef union ngtcp2_max_frame {
-  ngtcp2_frame fr;
-  struct {
-    ngtcp2_ack ack;
-    /* ack includes 1 ngtcp2_ack_range. */
-    ngtcp2_ack_range ranges[NGTCP2_MAX_ACK_RANGES - 1];
-  } ackfr;
-} ngtcp2_max_frame;
-
 typedef struct ngtcp2_path_challenge_entry {
   ngtcp2_path_storage ps;
   uint8_t data[8];
