@@ -233,7 +233,7 @@ static ngtcp2_ssize rtb_reclaim_frame(ngtcp2_rtb *rtb, uint8_t flags,
       continue;
     }
 
-    switch (frc->fr.type) {
+    switch (frc->fr.hd.type) {
     case NGTCP2_FRAME_STREAM:
       strm = ngtcp2_conn_find_stream(conn, fr->stream.stream_id);
       if (strm == NULL || (strm->flags & NGTCP2_STRM_FLAG_RESET_STREAM)) {
@@ -271,7 +271,12 @@ static ngtcp2_ssize rtb_reclaim_frame(ngtcp2_rtb *rtb, uint8_t flags,
         return rv;
       }
 
-      nfrc->fr = *fr;
+      nfrc->fr.stream.type = fr->stream.type;
+      nfrc->fr.stream.flags = fr->stream.flags;
+      nfrc->fr.stream.fin = fr->stream.fin;
+      nfrc->fr.stream.stream_id = fr->stream.stream_id;
+      nfrc->fr.stream.offset = fr->stream.offset;
+      nfrc->fr.stream.datacnt = fr->stream.datacnt;
       ngtcp2_vec_copy(nfrc->fr.stream.data, fr->stream.data,
                       fr->stream.datacnt);
 
@@ -314,7 +319,12 @@ static ngtcp2_ssize rtb_reclaim_frame(ngtcp2_rtb *rtb, uint8_t flags,
         return rv;
       }
 
-      nfrc->fr = *fr;
+      nfrc->fr.stream.type = fr->stream.type;
+      nfrc->fr.stream.flags = 0;
+      nfrc->fr.stream.fin = 0;
+      nfrc->fr.stream.stream_id = 0;
+      nfrc->fr.stream.offset = fr->stream.offset;
+      nfrc->fr.stream.datacnt = fr->stream.datacnt;
       ngtcp2_vec_copy(nfrc->fr.stream.data, fr->stream.data,
                       fr->stream.datacnt);
 
@@ -415,7 +425,7 @@ static int conn_process_lost_datagram(ngtcp2_conn *conn,
   int rv;
 
   for (frc = ent->frc; frc; frc = frc->next) {
-    switch (frc->fr.type) {
+    switch (frc->fr.hd.type) {
     case NGTCP2_FRAME_DATAGRAM:
     case NGTCP2_FRAME_DATAGRAM_LEN:
       assert(conn->callbacks.lost_datagram);
@@ -531,9 +541,10 @@ static void rtb_remove(ngtcp2_rtb *rtb, ngtcp2_ksl_it *it,
   int rv;
   (void)rv;
 
+  rtb_on_remove(rtb, ent, cstat);
+
   rv = ngtcp2_ksl_remove_hint(&rtb->ents, it, it, &ent->hd.pkt_num);
   assert(0 == rv);
-  rtb_on_remove(rtb, ent, cstat);
 
   assert(ent->next == NULL);
 
@@ -602,7 +613,7 @@ static int process_acked_pkt(ngtcp2_rtb_entry *ent, ngtcp2_conn *conn,
       frc->binder->flags |= NGTCP2_FRAME_CHAIN_BINDER_FLAG_ACK;
     }
 
-    switch (frc->fr.type) {
+    switch (frc->fr.hd.type) {
     case NGTCP2_FRAME_STREAM:
       strm = ngtcp2_conn_find_stream(conn, frc->fr.stream.stream_id);
       if (strm == NULL) {
@@ -850,7 +861,7 @@ ngtcp2_ssize ngtcp2_rtb_recv_ack(ngtcp2_rtb *rtb, const ngtcp2_ack *fr,
       goto fail;
     }
 
-    if (largest_ack == pkt_num) {
+    if (rtb->largest_acked_tx_pkt_num == pkt_num) {
       cc_ack.largest_pkt_sent_ts = ent->ts;
     }
 
@@ -895,11 +906,7 @@ ngtcp2_ssize ngtcp2_rtb_recv_ack(ngtcp2_rtb *rtb, const ngtcp2_ack *fr,
     cc_ack.rtt = ngtcp2_max_uint64(pkt_ts - cc_ack.largest_pkt_sent_ts,
                                    NGTCP2_NANOSECONDS);
 
-    rv = ngtcp2_conn_update_rtt(conn, cc_ack.rtt, fr->ack_delay_unscaled, ts);
-    if (rv == 0 && cc->new_rtt_sample &&
-        rtb->largest_acked_tx_pkt_num >= rtb->cc_pkt_num) {
-      cc->new_rtt_sample(cc, cstat, ts);
-    }
+    ngtcp2_conn_update_rtt(conn, cc_ack.rtt, fr->ack_delay_unscaled, ts);
   }
 
   if (conn) {
@@ -1193,12 +1200,12 @@ static int rtb_detect_lost_pkt(ngtcp2_rtb *rtb, ngtcp2_cc_ack *cc_ack,
 int ngtcp2_rtb_detect_lost_pkt(ngtcp2_rtb *rtb, ngtcp2_conn *conn,
                                ngtcp2_pktns *pktns, ngtcp2_conn_stat *cstat,
                                ngtcp2_tstamp ts) {
-  ngtcp2_cc_ack cc_ack = {
-    .largest_pkt_sent_ts = UINT64_MAX,
-    .rtt = UINT64_MAX,
-  };
-
-  return rtb_detect_lost_pkt(rtb, &cc_ack, conn, pktns, cstat, ts);
+  return rtb_detect_lost_pkt(rtb,
+                             &(ngtcp2_cc_ack){
+                               .largest_pkt_sent_ts = UINT64_MAX,
+                               .rtt = UINT64_MAX,
+                             },
+                             conn, pktns, cstat, ts);
 }
 
 void ngtcp2_rtb_remove_excessive_lost_pkt(ngtcp2_rtb *rtb, size_t n) {
@@ -1251,8 +1258,9 @@ void ngtcp2_rtb_remove_expired_lost_pkt(ngtcp2_rtb *rtb,
     ngtcp2_ksl_it_prev(&it);
     ent = ngtcp2_ksl_it_get(&it);
 
-    if (!(ent->flags & NGTCP2_RTB_ENTRY_FLAG_LOST_RETRANSMITTED) ||
-        ts - ent->lost_ts < timeout) {
+    assert(ent->flags & NGTCP2_RTB_ENTRY_FLAG_LOST_RETRANSMITTED);
+
+    if (ts - ent->lost_ts < timeout) {
       return;
     }
 
@@ -1301,7 +1309,7 @@ static int rtb_reclaim_frame_on_retry(ngtcp2_rtb *rtb, ngtcp2_conn *conn,
   int rv;
 
   for (; *pfrc;) {
-    switch ((*pfrc)->fr.type) {
+    switch ((*pfrc)->fr.hd.type) {
     case NGTCP2_FRAME_STREAM:
       frc = *pfrc;
 
@@ -1412,12 +1420,19 @@ int ngtcp2_rtb_reclaim_on_retry(ngtcp2_rtb *rtb, ngtcp2_conn *conn,
       ngtcp2_log_infof(rtb->log, NGTCP2_LOG_EVENT_LDC,
                        "pkn=%" PRId64 " has already been reclaimed on PTO",
                        ent->hd.pkt_num);
+
+      ngtcp2_rtb_entry_objalloc_del(ent, rtb->rtb_entry_objalloc,
+                                    rtb->frc_objalloc, rtb->mem);
+
       continue;
     }
 
     if (!(ent->flags & NGTCP2_RTB_ENTRY_FLAG_RETRANSMITTABLE) &&
         (!(ent->flags & NGTCP2_RTB_ENTRY_FLAG_DATAGRAM) ||
          !conn->callbacks.lost_datagram)) {
+      ngtcp2_rtb_entry_objalloc_del(ent, rtb->rtb_entry_objalloc,
+                                    rtb->frc_objalloc, rtb->mem);
+
       continue;
     }
 
