@@ -48,8 +48,10 @@ const MunitSuite rtb_suite = {
 };
 
 static void conn_stat_init(ngtcp2_conn_stat *cstat) {
-  memset(cstat, 0, sizeof(*cstat));
-  cstat->max_tx_udp_payload_size = NGTCP2_MAX_UDP_PAYLOAD_SIZE;
+  *cstat = (ngtcp2_conn_stat){
+    .cwnd = 10 * NGTCP2_MAX_UDP_PAYLOAD_SIZE,
+    .max_tx_udp_payload_size = NGTCP2_MAX_UDP_PAYLOAD_SIZE,
+  };
 }
 
 void test_ngtcp2_rtb_add(void) {
@@ -74,7 +76,7 @@ void test_ngtcp2_rtb_add(void) {
   conn_stat_init(&cstat);
   ngtcp2_rst_init(&rst);
   ngtcp2_log_init(&log, NULL, NULL, 0, NULL);
-  ngtcp2_cc_reno_init(&cc, &log);
+  ngtcp2_cc_reno_init(&cc, &log, &cstat);
   ngtcp2_rtb_init(&rtb, &rst, &cc.cc, 0, &log, NULL, &rtb_entry_objalloc,
                   &frc_objalloc, mem);
 
@@ -184,9 +186,8 @@ static void assert_rtb_entry_not_found(ngtcp2_rtb *rtb, int64_t pkt_num) {
 void test_ngtcp2_rtb_recv_ack(void) {
   ngtcp2_rtb rtb;
   const ngtcp2_mem *mem = ngtcp2_mem_default();
-  ngtcp2_max_frame mfr;
-  ngtcp2_ack *fr = &mfr.ackfr.ack;
-  ngtcp2_ack_range *ranges;
+  ngtcp2_ack_range ack_ranges[NGTCP2_MAX_ACK_RANGES];
+  ngtcp2_ack fr;
   ngtcp2_log log;
   ngtcp2_conn_stat cstat;
   ngtcp2_cc_reno cc;
@@ -208,20 +209,20 @@ void test_ngtcp2_rtb_recv_ack(void) {
   /* no ack block */
   conn_stat_init(&cstat);
   ngtcp2_rst_init(&rst);
-  ngtcp2_cc_reno_init(&cc, &log);
+  ngtcp2_cc_reno_init(&cc, &log, &cstat);
   ngtcp2_rtb_init(&rtb, &rst, &cc.cc, 0, &log, NULL, &rtb_entry_objalloc,
                   &frc_objalloc, mem);
   setup_rtb_fixture(&rtb, &cstat, &rtb_entry_objalloc);
 
   assert_size(67, ==, ngtcp2_ksl_len(&rtb.ents));
 
-  *fr = (ngtcp2_ack){
+  fr = (ngtcp2_ack){
     .largest_ack = 446,
     .first_ack_range = 1,
   };
 
-  num_acked =
-    ngtcp2_rtb_recv_ack(&rtb, fr, &cstat, NULL, &pktns, 1000000009, 1000000009);
+  num_acked = ngtcp2_rtb_recv_ack(&rtb, &fr, &cstat, NULL, &pktns, 1000000009,
+                                  1000000009);
 
   assert_ptrdiff(2, ==, num_acked);
   assert_size(65, ==, ngtcp2_ksl_len(&rtb.ents));
@@ -232,25 +233,27 @@ void test_ngtcp2_rtb_recv_ack(void) {
 
   /* with ack block */
   conn_stat_init(&cstat);
-  ngtcp2_cc_reno_init(&cc, &log);
+  ngtcp2_cc_reno_init(&cc, &log, &cstat);
   ngtcp2_rtb_init(&rtb, &rst, &cc.cc, 0, &log, NULL, &rtb_entry_objalloc,
                   &frc_objalloc, mem);
   setup_rtb_fixture(&rtb, &cstat, &rtb_entry_objalloc);
 
-  *fr = (ngtcp2_ack){
+  fr = (ngtcp2_ack){
     .largest_ack = 441,
     .first_ack_range = 3, /* (441), (440), 439, 438 */
     .rangecnt = 2,
+    .ranges = ack_ranges,
   };
-  ranges = fr->ranges;
-  ranges[0].gap = 253; /* (183) */
-  ranges[1] = (ngtcp2_ack_range){
+  ack_ranges[0] = (ngtcp2_ack_range){
+    .gap = 253, /* (183) */
+  };
+  ack_ranges[1] = (ngtcp2_ack_range){
     .gap = 1, /* 182, 181 */
     .len = 1, /* (180), 179 */
   };
 
-  num_acked =
-    ngtcp2_rtb_recv_ack(&rtb, fr, &cstat, NULL, &pktns, 1000000009, 1000000009);
+  num_acked = ngtcp2_rtb_recv_ack(&rtb, &fr, &cstat, NULL, &pktns, 1000000009,
+                                  1000000009);
 
   assert_ptrdiff(4, ==, num_acked);
   assert_size(63, ==, ngtcp2_ksl_len(&rtb.ents));
@@ -264,19 +267,22 @@ void test_ngtcp2_rtb_recv_ack(void) {
 
   /* gap+len points to pkt_num 0 */
   conn_stat_init(&cstat);
-  ngtcp2_cc_reno_init(&cc, &log);
+  ngtcp2_cc_reno_init(&cc, &log, &cstat);
   ngtcp2_rtb_init(&rtb, &rst, &cc.cc, 0, &log, NULL, &rtb_entry_objalloc,
                   &frc_objalloc, mem);
   add_rtb_entry_range(&rtb, 0, 1, &cstat, &rtb_entry_objalloc);
 
-  *fr = (ngtcp2_ack){
+  fr = (ngtcp2_ack){
     .largest_ack = 250,
     .rangecnt = 1,
-    .ranges[0].gap = 248,
+    .ranges = ack_ranges,
+  };
+  ack_ranges[0] = (ngtcp2_ack_range){
+    .gap = 248,
   };
 
-  num_acked =
-    ngtcp2_rtb_recv_ack(&rtb, fr, &cstat, NULL, &pktns, 1000000009, 1000000009);
+  num_acked = ngtcp2_rtb_recv_ack(&rtb, &fr, &cstat, NULL, &pktns, 1000000009,
+                                  1000000009);
 
   assert_ptrdiff(1, ==, num_acked);
   assert_rtb_entry_not_found(&rtb, 0);
@@ -285,15 +291,15 @@ void test_ngtcp2_rtb_recv_ack(void) {
 
   /* pkt_num = 0 (first ack block) */
   conn_stat_init(&cstat);
-  ngtcp2_cc_reno_init(&cc, &log);
+  ngtcp2_cc_reno_init(&cc, &log, &cstat);
   ngtcp2_rtb_init(&rtb, &rst, &cc.cc, 0, &log, NULL, &rtb_entry_objalloc,
                   &frc_objalloc, mem);
   add_rtb_entry_range(&rtb, 0, 1, &cstat, &rtb_entry_objalloc);
 
-  *fr = (ngtcp2_ack){0};
+  fr = (ngtcp2_ack){0};
 
-  num_acked =
-    ngtcp2_rtb_recv_ack(&rtb, fr, &cstat, NULL, &pktns, 1000000009, 1000000009);
+  num_acked = ngtcp2_rtb_recv_ack(&rtb, &fr, &cstat, NULL, &pktns, 1000000009,
+                                  1000000009);
 
   assert_ptrdiff(1, ==, num_acked);
   assert_rtb_entry_not_found(&rtb, 0);
@@ -302,18 +308,20 @@ void test_ngtcp2_rtb_recv_ack(void) {
 
   /* pkt_num = 0 */
   conn_stat_init(&cstat);
-  ngtcp2_cc_reno_init(&cc, &log);
+  ngtcp2_cc_reno_init(&cc, &log, &cstat);
   ngtcp2_rtb_init(&rtb, &rst, &cc.cc, 0, &log, NULL, &rtb_entry_objalloc,
                   &frc_objalloc, mem);
   add_rtb_entry_range(&rtb, 0, 1, &cstat, &rtb_entry_objalloc);
 
-  *fr = (ngtcp2_ack){
+  fr = (ngtcp2_ack){
     .largest_ack = 2,
     .rangecnt = 1,
+    .ranges = ack_ranges,
   };
+  ack_ranges[0] = (ngtcp2_ack_range){0};
 
-  num_acked =
-    ngtcp2_rtb_recv_ack(&rtb, fr, &cstat, NULL, &pktns, 1000000009, 1000000009);
+  num_acked = ngtcp2_rtb_recv_ack(&rtb, &fr, &cstat, NULL, &pktns, 1000000009,
+                                  1000000009);
 
   assert_ptrdiff(1, ==, num_acked);
   assert_rtb_entry_not_found(&rtb, 0);
@@ -322,16 +330,16 @@ void test_ngtcp2_rtb_recv_ack(void) {
 
   /* acknowledging skipped packet number in the first block */
   conn_stat_init(&cstat);
-  ngtcp2_cc_reno_init(&cc, &log);
+  ngtcp2_cc_reno_init(&cc, &log, &cstat);
   ngtcp2_rtb_init(&rtb, &rst, &cc.cc, 0, &log, NULL, &rtb_entry_objalloc,
                   &frc_objalloc, mem);
   add_rtb_entry_range_with_flags(&rtb, 0, 1, NGTCP2_RTB_ENTRY_FLAG_SKIP, &cstat,
                                  &rtb_entry_objalloc);
 
-  *fr = (ngtcp2_ack){0};
+  fr = (ngtcp2_ack){0};
 
-  num_acked =
-    ngtcp2_rtb_recv_ack(&rtb, fr, &cstat, NULL, &pktns, 1000000009, 1000000009);
+  num_acked = ngtcp2_rtb_recv_ack(&rtb, &fr, &cstat, NULL, &pktns, 1000000009,
+                                  1000000009);
 
   assert_ptrdiff(NGTCP2_ERR_PROTO, ==, num_acked);
 
@@ -339,19 +347,21 @@ void test_ngtcp2_rtb_recv_ack(void) {
 
   /* acknowledging skipped packet number in the second block */
   conn_stat_init(&cstat);
-  ngtcp2_cc_reno_init(&cc, &log);
+  ngtcp2_cc_reno_init(&cc, &log, &cstat);
   ngtcp2_rtb_init(&rtb, &rst, &cc.cc, 0, &log, NULL, &rtb_entry_objalloc,
                   &frc_objalloc, mem);
   add_rtb_entry_range_with_flags(&rtb, 0, 1, NGTCP2_RTB_ENTRY_FLAG_SKIP, &cstat,
                                  &rtb_entry_objalloc);
 
-  *fr = (ngtcp2_ack){
+  fr = (ngtcp2_ack){
     .largest_ack = 2,
     .rangecnt = 1,
+    .ranges = ack_ranges,
   };
+  ack_ranges[0] = (ngtcp2_ack_range){0};
 
-  num_acked =
-    ngtcp2_rtb_recv_ack(&rtb, fr, &cstat, NULL, &pktns, 1000000009, 1000000009);
+  num_acked = ngtcp2_rtb_recv_ack(&rtb, &fr, &cstat, NULL, &pktns, 1000000009,
+                                  1000000009);
 
   assert_ptrdiff(NGTCP2_ERR_PROTO, ==, num_acked);
 
@@ -380,7 +390,7 @@ void test_ngtcp2_rtb_lost_pkt_ts(void) {
 
   conn_stat_init(&cstat);
   ngtcp2_rst_init(&rst);
-  ngtcp2_cc_reno_init(&cc, &log);
+  ngtcp2_cc_reno_init(&cc, &log, &cstat);
   ngtcp2_rtb_init(&rtb, &rst, &cc.cc, 0, &log, NULL, &rtb_entry_objalloc,
                   &frc_objalloc, mem);
 
@@ -422,7 +432,7 @@ void test_ngtcp2_rtb_remove_expired_lost_pkt(void) {
 
   conn_stat_init(&cstat);
   ngtcp2_rst_init(&rst);
-  ngtcp2_cc_reno_init(&cc, &log);
+  ngtcp2_cc_reno_init(&cc, &log, &cstat);
   ngtcp2_rtb_init(&rtb, &rst, &cc.cc, 0, &log, NULL, &rtb_entry_objalloc,
                   &frc_objalloc, mem);
 
@@ -483,7 +493,7 @@ void test_ngtcp2_rtb_remove_excessive_lost_pkt(void) {
 
   conn_stat_init(&cstat);
   ngtcp2_rst_init(&rst);
-  ngtcp2_cc_reno_init(&cc, &log);
+  ngtcp2_cc_reno_init(&cc, &log, &cstat);
   ngtcp2_rtb_init(&rtb, &rst, &cc.cc, 0, &log, NULL, &rtb_entry_objalloc,
                   &frc_objalloc, mem);
 
