@@ -56,10 +56,12 @@ static void setup_strm_streamfrq_fixture(ngtcp2_strm *strm,
                    mem);
 
   ngtcp2_frame_chain_stream_datacnt_objalloc_new(&frc, 2, frc_objalloc, mem);
-  frc->fr.stream = (ngtcp2_stream){
-    .type = NGTCP2_FRAME_STREAM,
-    .datacnt = 2,
-  };
+  frc->fr.stream.type = NGTCP2_FRAME_STREAM;
+  frc->fr.stream.flags = 0;
+  frc->fr.stream.fin = 0;
+  frc->fr.stream.stream_id = 0;
+  frc->fr.stream.offset = 0;
+  frc->fr.stream.datacnt = 2;
   data = frc->fr.stream.data;
   data[0] = (ngtcp2_vec){
     .len = 50,
@@ -73,11 +75,12 @@ static void setup_strm_streamfrq_fixture(ngtcp2_strm *strm,
   ngtcp2_strm_streamfrq_push(strm, frc);
 
   ngtcp2_frame_chain_stream_datacnt_objalloc_new(&frc, 2, frc_objalloc, mem);
-  frc->fr.stream = (ngtcp2_stream){
-    .type = NGTCP2_FRAME_STREAM,
-    .offset = 128,
-    .datacnt = 2,
-  };
+  frc->fr.stream.type = NGTCP2_FRAME_STREAM;
+  frc->fr.stream.flags = 0;
+  frc->fr.stream.fin = 0;
+  frc->fr.stream.stream_id = 0;
+  frc->fr.stream.offset = 128;
+  frc->fr.stream.datacnt = 2;
   data = frc->fr.stream.data;
   data[0] = (ngtcp2_vec){
     .len = 60,
@@ -91,11 +94,12 @@ static void setup_strm_streamfrq_fixture(ngtcp2_strm *strm,
   ngtcp2_strm_streamfrq_push(strm, frc);
 
   ngtcp2_frame_chain_stream_datacnt_objalloc_new(&frc, 2, frc_objalloc, mem);
-  frc->fr.stream = (ngtcp2_stream){
-    .type = NGTCP2_FRAME_STREAM,
-    .offset = 272,
-    .datacnt = 2,
-  };
+  frc->fr.stream.type = NGTCP2_FRAME_STREAM;
+  frc->fr.stream.flags = 0;
+  frc->fr.stream.fin = 0;
+  frc->fr.stream.stream_id = 0;
+  frc->fr.stream.offset = 272;
+  frc->fr.stream.datacnt = 2;
   data = frc->fr.stream.data;
   data[0] = (ngtcp2_vec){
     .len = 127,
@@ -267,28 +271,29 @@ void test_ngtcp2_strm_streamfrq_pop(void) {
                    mem);
 
   ngtcp2_frame_chain_stream_datacnt_objalloc_new(&frc, 1, &frc_objalloc, mem);
-  frc->fr.stream = (ngtcp2_stream){
-    .type = NGTCP2_FRAME_STREAM,
-    .datacnt = 1,
-    .data[0] =
-      {
-        .len = 11,
-        .base = nulldata,
-      },
+  frc->fr.stream.type = NGTCP2_FRAME_STREAM;
+  frc->fr.stream.flags = 0;
+  frc->fr.stream.fin = 0;
+  frc->fr.stream.stream_id = 0;
+  frc->fr.stream.offset = 0;
+  frc->fr.stream.datacnt = 1;
+  frc->fr.stream.data[0] = (ngtcp2_vec){
+    .len = 11,
+    .base = nulldata,
   };
 
   ngtcp2_strm_streamfrq_push(&strm, frc);
 
   ngtcp2_frame_chain_stream_datacnt_objalloc_new(&frc, 1, &frc_objalloc, mem);
-  frc->fr.stream = (ngtcp2_stream){
-    .type = NGTCP2_FRAME_STREAM,
-    .offset = 30,
-    .datacnt = 1,
-    .data[0] =
-      {
-        .len = 17,
-        .base = nulldata + 30,
-      },
+  frc->fr.stream.type = NGTCP2_FRAME_STREAM;
+  frc->fr.stream.flags = 0;
+  frc->fr.stream.fin = 0;
+  frc->fr.stream.stream_id = 0;
+  frc->fr.stream.offset = 30;
+  frc->fr.stream.datacnt = 1;
+  frc->fr.stream.data[0] = (ngtcp2_vec){
+    .len = 17,
+    .base = nulldata + 30,
   };
 
   ngtcp2_strm_streamfrq_push(&strm, frc);
@@ -309,24 +314,24 @@ void test_ngtcp2_strm_streamfrq_pop(void) {
                    mem);
 
   ngtcp2_frame_chain_stream_datacnt_objalloc_new(&frc, 1, &frc_objalloc, mem);
-  frc->fr.stream = (ngtcp2_stream){
-    .type = NGTCP2_FRAME_STREAM,
-    .datacnt = 1,
-    .data[0] =
-      {
-        .len = 11,
-        .base = nulldata,
-      },
+  frc->fr.stream.type = NGTCP2_FRAME_STREAM;
+  frc->fr.stream.flags = 0;
+  frc->fr.stream.fin = 0;
+  frc->fr.stream.stream_id = 0;
+  frc->fr.stream.datacnt = 1;
+  frc->fr.stream.data[0] = (ngtcp2_vec){
+    .len = 11,
+    .base = nulldata,
   };
 
   ngtcp2_strm_streamfrq_push(&strm, frc);
 
   ngtcp2_frame_chain_stream_datacnt_objalloc_new(&frc, 0, &frc_objalloc, mem);
-  frc->fr.stream = (ngtcp2_stream){
-    .type = NGTCP2_FRAME_STREAM,
-    .fin = 1,
-    .offset = 11,
-  };
+  frc->fr.stream.type = NGTCP2_FRAME_STREAM;
+  frc->fr.stream.flags = 0;
+  frc->fr.stream.fin = 1;
+  frc->fr.stream.stream_id = 0;
+  frc->fr.stream.offset = 11;
 
   ngtcp2_strm_streamfrq_push(&strm, frc);
 
@@ -355,25 +360,28 @@ void test_ngtcp2_strm_streamfrq_pop(void) {
   ngtcp2_strm_init(&strm, 0, NGTCP2_STRM_FLAG_NONE, 0, 0, NULL, &frc_objalloc,
                    mem);
   ngtcp2_frame_chain_stream_datacnt_objalloc_new(&frc, 1, &frc_objalloc, mem);
-  frc->fr.stream = (ngtcp2_stream){
-    .type = NGTCP2_FRAME_STREAM,
-    .datacnt = 1,
-    .data[0] =
-      {
-        .len = 17,
-        .base = nulldata,
-      },
+  frc->fr.stream.type = NGTCP2_FRAME_STREAM;
+  frc->fr.stream.flags = 0;
+  frc->fr.stream.fin = 0;
+  frc->fr.stream.stream_id = 0;
+  frc->fr.stream.offset = 0;
+  frc->fr.stream.datacnt = 1;
+  frc->fr.stream.data[0] = (ngtcp2_vec){
+    .len = 17,
+    .base = nulldata,
   };
 
   ngtcp2_strm_streamfrq_push(&strm, frc);
 
   ngtcp2_frame_chain_stream_datacnt_objalloc_new(
     &frc, NGTCP2_FRAME_CHAIN_STREAM_DATACNT_THRES + 1, &frc_objalloc, mem);
-  frc->fr.stream = (ngtcp2_stream){
-    .type = NGTCP2_FRAME_STREAM,
-    .offset = 17,
-    .datacnt = NGTCP2_FRAME_CHAIN_STREAM_DATACNT_THRES + 1,
-  };
+  frc->fr.stream.type = NGTCP2_FRAME_STREAM;
+  frc->fr.stream.flags = 0;
+  frc->fr.stream.fin = 0;
+  frc->fr.stream.stream_id = 0;
+  frc->fr.stream.offset = 17;
+  frc->fr.stream.datacnt = NGTCP2_FRAME_CHAIN_STREAM_DATACNT_THRES + 1;
+
   data = frc->fr.stream.data;
 
   for (i = 0; i < frc->fr.stream.datacnt; ++i) {
@@ -431,29 +439,29 @@ void test_ngtcp2_strm_streamfrq_unacked_offset(void) {
                    &frc_objalloc, mem);
 
   ngtcp2_frame_chain_stream_datacnt_objalloc_new(&frc, 1, &frc_objalloc, mem);
-  frc->fr.stream = (ngtcp2_stream){
-    .type = NGTCP2_FRAME_STREAM,
-    .datacnt = 1,
-    .data[0] =
-      {
-        .len = 17,
-        .base = nulldata,
-      },
+  frc->fr.stream.type = NGTCP2_FRAME_STREAM;
+  frc->fr.stream.flags = 0;
+  frc->fr.stream.fin = 0;
+  frc->fr.stream.stream_id = 0;
+  frc->fr.stream.offset = 0;
+  frc->fr.stream.datacnt = 1;
+  frc->fr.stream.data[0] = (ngtcp2_vec){
+    .len = 17,
+    .base = nulldata,
   };
 
   ngtcp2_strm_streamfrq_push(&strm, frc);
 
   ngtcp2_frame_chain_stream_datacnt_objalloc_new(&frc, 1, &frc_objalloc, mem);
-  frc->fr.stream = (ngtcp2_stream){
-    .type = NGTCP2_FRAME_STREAM,
-    .fin = 1,
-    .offset = 443,
-    .datacnt = 1,
-    .data[0] =
-      {
-        .len = 971,
-        .base = nulldata,
-      },
+  frc->fr.stream.type = NGTCP2_FRAME_STREAM;
+  frc->fr.stream.flags = 0;
+  frc->fr.stream.fin = 1;
+  frc->fr.stream.stream_id = 0;
+  frc->fr.stream.offset = 443;
+  frc->fr.stream.datacnt = 1;
+  frc->fr.stream.data[0] = (ngtcp2_vec){
+    .len = 971,
+    .base = nulldata,
   };
 
   ngtcp2_strm_streamfrq_push(&strm, frc);
@@ -469,29 +477,29 @@ void test_ngtcp2_strm_streamfrq_unacked_offset(void) {
                    mem);
 
   ngtcp2_frame_chain_stream_datacnt_objalloc_new(&frc, 1, &frc_objalloc, mem);
-  frc->fr.stream = (ngtcp2_stream){
-    .type = NGTCP2_FRAME_STREAM,
-    .datacnt = 1,
-    .data[0] =
-      {
-        .len = 17,
-        .base = nulldata,
-      },
+  frc->fr.stream.type = NGTCP2_FRAME_STREAM;
+  frc->fr.stream.flags = 0;
+  frc->fr.stream.fin = 0;
+  frc->fr.stream.stream_id = 0;
+  frc->fr.stream.offset = 0;
+  frc->fr.stream.datacnt = 1;
+  frc->fr.stream.data[0] = (ngtcp2_vec){
+    .len = 17,
+    .base = nulldata,
   };
 
   ngtcp2_strm_streamfrq_push(&strm, frc);
 
   ngtcp2_frame_chain_stream_datacnt_objalloc_new(&frc, 1, &frc_objalloc, mem);
-  frc->fr.stream = (ngtcp2_stream){
-    .type = NGTCP2_FRAME_STREAM,
-    .fin = 1,
-    .offset = 443,
-    .datacnt = 1,
-    .data[0] =
-      {
-        .len = 971,
-        .base = nulldata,
-      },
+  frc->fr.stream.type = NGTCP2_FRAME_STREAM;
+  frc->fr.stream.flags = 0;
+  frc->fr.stream.fin = 1;
+  frc->fr.stream.stream_id = 0;
+  frc->fr.stream.offset = 443;
+  frc->fr.stream.datacnt = 1;
+  frc->fr.stream.data[0] = (ngtcp2_vec){
+    .len = 971,
+    .base = nulldata,
   };
 
   ngtcp2_strm_streamfrq_push(&strm, frc);
@@ -507,14 +515,15 @@ void test_ngtcp2_strm_streamfrq_unacked_offset(void) {
                    mem);
 
   ngtcp2_frame_chain_stream_datacnt_objalloc_new(&frc, 1, &frc_objalloc, mem);
-  frc->fr.stream = (ngtcp2_stream){
-    .type = NGTCP2_FRAME_STREAM,
-    .datacnt = 1,
-    .data[0] =
-      {
-        .len = 971,
-        .base = nulldata,
-      },
+  frc->fr.stream.type = NGTCP2_FRAME_STREAM;
+  frc->fr.stream.flags = 0;
+  frc->fr.stream.fin = 0;
+  frc->fr.stream.stream_id = 0;
+  frc->fr.stream.offset = 0;
+  frc->fr.stream.datacnt = 1;
+  frc->fr.stream.data[0] = (ngtcp2_vec){
+    .len = 971,
+    .base = nulldata,
   };
 
   ngtcp2_strm_streamfrq_push(&strm, frc);
@@ -530,14 +539,15 @@ void test_ngtcp2_strm_streamfrq_unacked_offset(void) {
                    mem);
 
   ngtcp2_frame_chain_stream_datacnt_objalloc_new(&frc, 1, &frc_objalloc, mem);
-  frc->fr.stream = (ngtcp2_stream){
-    .type = NGTCP2_FRAME_STREAM,
-    .datacnt = 1,
-    .data[0] =
-      {
-        .len = 971,
-        .base = nulldata,
-      },
+  frc->fr.stream.type = NGTCP2_FRAME_STREAM;
+  frc->fr.stream.flags = 0;
+  frc->fr.stream.fin = 0;
+  frc->fr.stream.stream_id = 0;
+  frc->fr.stream.offset = 0;
+  frc->fr.stream.datacnt = 1;
+  frc->fr.stream.data[0] = (ngtcp2_vec){
+    .len = 971,
+    .base = nulldata,
   };
 
   ngtcp2_strm_streamfrq_push(&strm, frc);
@@ -553,15 +563,15 @@ void test_ngtcp2_strm_streamfrq_unacked_offset(void) {
                    mem);
 
   ngtcp2_frame_chain_stream_datacnt_objalloc_new(&frc, 1, &frc_objalloc, mem);
-  frc->fr.stream = (ngtcp2_stream){
-    .type = NGTCP2_FRAME_STREAM,
-    .offset = 977,
-    .datacnt = 1,
-    .data[0] =
-      {
-        .len = 971,
-        .base = nulldata,
-      },
+  frc->fr.stream.type = NGTCP2_FRAME_STREAM;
+  frc->fr.stream.flags = 0;
+  frc->fr.stream.fin = 0;
+  frc->fr.stream.stream_id = 0;
+  frc->fr.stream.offset = 977;
+  frc->fr.stream.datacnt = 1;
+  frc->fr.stream.data[0] = (ngtcp2_vec){
+    .len = 971,
+    .base = nulldata,
   };
 
   ngtcp2_strm_streamfrq_push(&strm, frc);
@@ -653,7 +663,7 @@ void test_ngtcp2_strm_streamfrq_unacked_pop(void) {
   rv = ngtcp2_strm_streamfrq_pop(&strm, &frc, 1024);
 
   assert_int(0, ==, rv);
-  assert_uint64(NGTCP2_FRAME_STREAM, ==, frc->fr.type);
+  assert_uint64(NGTCP2_FRAME_STREAM, ==, frc->fr.hd.type);
   assert_true(frc->fr.stream.fin);
   assert_uint64(764, ==, frc->fr.stream.offset);
   assert_uint64(0, ==,
@@ -670,7 +680,7 @@ void test_ngtcp2_strm_streamfrq_unacked_pop(void) {
   rv = ngtcp2_strm_streamfrq_pop(&strm, &frc, 140);
 
   assert_int(0, ==, rv);
-  assert_uint64(NGTCP2_FRAME_STREAM, ==, frc->fr.type);
+  assert_uint64(NGTCP2_FRAME_STREAM, ==, frc->fr.hd.type);
   assert_false(frc->fr.stream.fin);
   assert_uint64(12, ==, frc->fr.stream.offset);
   assert_size(2, ==, frc->fr.stream.datacnt);
@@ -689,7 +699,7 @@ void test_ngtcp2_strm_streamfrq_unacked_pop(void) {
   rv = ngtcp2_strm_streamfrq_pop(&strm, &frc, 140);
 
   assert_int(0, ==, rv);
-  assert_uint64(NGTCP2_FRAME_STREAM, ==, frc->fr.type);
+  assert_uint64(NGTCP2_FRAME_STREAM, ==, frc->fr.hd.type);
   assert_false(frc->fr.stream.fin);
   assert_uint64(0, ==, frc->fr.stream.offset);
   assert_size(2, ==, frc->fr.stream.datacnt);
